@@ -1,6 +1,6 @@
 package String::Approx;
 
-$VERSION = 3.14;
+$VERSION = '3.24';
 
 use strict;
 local $^W = 1;
@@ -18,32 +18,76 @@ require DynaLoader;
 
 bootstrap String::Approx $VERSION;
 
-my $CACHE_MAX	= 1000;		# high water mark
-my $CACHE_PURGE	= 0.75;		# purge this much of the least used
-my $CACHE_N_PURGE = $CACHE_MAX * $CACHE_PURGE;
+my $CACHE_MAX = 1000;	# high water mark
+my $CACHE_PURGE = 0.75;	# purge this much of the least used
+my $CACHE_N_PURGE;	# purge this many of the least used
+
+sub cache_n_purge () {
+    $CACHE_N_PURGE = $CACHE_MAX * $CACHE_PURGE;
+    $CACHE_N_PURGE = 1 if $CACHE_N_PURGE < 1;
+    return $CACHE_N_PURGE;
+}
+
+cache_n_purge();
+
+sub cache_max (;$) {
+    if (@_ == 0) {
+	return $CACHE_MAX;
+    } else {
+	$CACHE_MAX = shift;
+    }
+    $CACHE_MAX = 0 if $CACHE_MAX < 0;
+    cache_n_purge();
+}
+
+sub cache_purge (;$) {
+    if (@_ == 0) {
+	return $CACHE_PURGE;
+    } else {
+	$CACHE_PURGE = shift;
+    }
+    if ($CACHE_PURGE < 0) {
+	$CACHE_PURGE = 0;
+    } elsif ($CACHE_PURGE > 1) {
+	$CACHE_PURGE = 1;
+    }
+    cache_n_purge();
+}
 
 my %_simple;
 my %_simple_usage_count;
 
+sub _cf_simple {
+    my $P = shift;
+
+    my @usage =
+	sort { $_simple_usage_count{$a} <=> $_simple_usage_count{$b} }
+             grep { $_ ne $P }
+                  keys %_simple_usage_count;
+	    
+    # Make room, delete the least used entries.
+    $#usage = $CACHE_N_PURGE - 1;
+	    
+    delete @_simple_usage_count{@usage};
+    delete @_simple{@usage};
+}
+
 sub _simple {
     my $P = shift;
 
-    $_simple{$P} = new(__PACKAGE__, $P) unless exists $_simple{$P};
+    my $_simple = new(__PACKAGE__, $P);
 
-    $_simple_usage_count{$P}++;
+    if ($CACHE_MAX) {
+	$_simple{$P} = $_simple unless exists $_simple{$P};
 
-    if (scalar keys %_simple_usage_count > $CACHE_MAX) {
-	my @usage =
-                sort { $_simple_usage_count{$a} <=> $_simple_usage_count{$b} }
-	             keys %_simple_usage_count;
+	$_simple_usage_count{$P}++;
 
-	foreach my $i (0..$CACHE_N_PURGE) {
-	    delete $_simple_usage_count{$usage[$i]}
-	        unless $usage[$i] ne $P;
+	if (keys %_simple_usage_count > $CACHE_MAX) {
+	    _cf_simple($P);
 	}
     }
 
-    return ( $_simple{$P} );
+    return ( $_simple );
 }
 
 sub _parse_param {
@@ -56,7 +100,7 @@ sub _parse_param {
         while ($_ ne '') {
 	    s/^\s+//;
             if (s/^([IDS]\s*)?(\d+)(\s*%)?//) {
-                my $k = defined $3 ? (($2-1) * $n) / 100 + 1 : $2;
+                my $k = defined $3 ? (($2-1) * $n) / 100 + ($2 ? 1 : 0) : $2;
 
 		if (defined $1) {
 		    $param{$1} = $k;
@@ -92,12 +136,30 @@ my %_parsed_param;
 my %_complex;
 my %_complex_usage_count;
 
+sub _cf_complex {
+    my $P = shift;
+
+    my @usage =
+	sort { $_complex_usage_count{$a} <=>
+		   $_complex_usage_count{$b} }
+             grep { $_ ne $P }
+                  keys %_complex_usage_count;
+	    
+    # Make room, delete the least used entries.
+    $#usage = $CACHE_N_PURGE - 1;
+	    
+    delete @_complex_usage_count{@usage};
+    delete @_complex{@usage};
+}
+
 sub _complex {
     my ($P, @param) = @_;
     unshift @param, length $P;
     my $param = "@param";
     my $_param_key;
     my %param;
+    my $complex;
+    my $is_new;
 
     unless (exists $_param_key{$param}) {
 	%param = _parse_param(@param);
@@ -109,59 +171,70 @@ sub _complex {
 
     $_param_key = $_param_key{$param};
 
-    unless (exists $_complex{$P}->{$_param_key}) {
-	$_complex{$P}->{$_param_key} = new(__PACKAGE__, $P)
-	    unless exists $_complex{$P}->{$_param_key};
-
-	$_complex{$P}->{$_param_key}->set_greedy unless exists $param{'?'};
-
-	if (exists $param{'k'}) {
-	    $_complex{$P}->{$_param_key} = new(__PACKAGE__, $P, $param{k});
-	} else {
-	    $_complex{$P}->{$_param_key} = new(__PACKAGE__, $P);
+    if ($CACHE_MAX) {
+	if (exists $_complex{$P}->{$_param_key}) {
+	    $complex = $_complex{$P}->{$_param_key};
 	}
+    }
 
-	$_complex{$P}->{$_param_key}->set_insertions($param{'I'})
+    unless (defined $complex) {
+	if (exists $param{'k'}) {
+	    $complex = new(__PACKAGE__, $P, $param{k});
+	} else {
+	    $complex = new(__PACKAGE__, $P);
+	}
+	$_complex{$P}->{$_param_key} = $complex if $CACHE_MAX;
+	$is_new = 1;
+    }
+
+    if ($is_new) {
+	$complex->set_greedy unless exists $param{'?'};
+
+	$complex->set_insertions($param{'I'})
 	    if exists $param{'I'};
-	$_complex{$P}->{$_param_key}->set_deletions($param{'D'})
+	$complex->set_deletions($param{'D'})
 	    if exists $param{'D'};
-	$_complex{$P}->{$_param_key}->set_substitutions($param{'S'})
+	$complex->set_substitutions($param{'S'})
 	    if exists $param{'S'};
 	
-	$_complex{$P}->{$_param_key}->set_caseignore_slice
+	$complex->set_caseignore_slice
 	    if exists $param{'i'};
 
-	$_complex{$P}->{$_param_key}->
-	  set_text_initial_position($param{'initial_position'})
+	$complex->set_text_initial_position($param{'initial_position'})
 	    if exists $param{'initial_position'};
 
-	$_complex{$P}->{$_param_key}->
-	  set_text_final_position($param{'final_position'})
+	$complex->set_text_final_position($param{'final_position'})
 	    if exists $param{'final_position'};
 
-	$_complex{$P}->{$_param_key}->
-	  set_text_position_range($param{'position_range'})
+	$complex->set_text_position_range($param{'position_range'})
 	    if exists $param{'position_range'};
 
-	$_complex{$P}->{$_param_key}->set_minimal_distance()
+	$complex->set_minimal_distance($param{'minimal_distance'})
 	    if exists $param{'minimal_distance'};
     }
 
-    $_complex_usage_count{$P}->{$_param_key}++;
+    if ($CACHE_MAX) {
+	$_complex_usage_count{$P}->{$_param_key}++;
 
-    # If our cache overfloweth.
-    if (scalar keys %_complex_usage_count > $CACHE_MAX) {
-	my @usage =
-                sort { $_complex_usage_count{$a} <=>
-                       $_complex_usage_count{$b} }
-	             keys %_complex_usage_count;
-	# Make room, delete the least used entries.
-	foreach my $i (0..$CACHE_N_PURGE) {
-	    delete $_complex_usage_count{$usage[$i]};
+	# If our cache overfloweth.
+	if (scalar keys %_complex_usage_count > $CACHE_MAX) {
+	    _cf_complex($P);
 	}
     }
 
-    return ( $_complex{$P}->{$_param_key}, %param );
+    return ( $complex, %param );
+}
+
+sub cache_disable {
+    cache_max(0);
+}
+
+sub cache_flush_all {
+    my $old_purge = cache_purge();
+    cache_purge(1);
+    _cf_simple('');
+    _cf_complex('');
+    cache_purge($old_purge);
 }
 
 sub amatch {
@@ -230,18 +303,6 @@ sub asubstitute {
 	(@_ && ref $_[0] eq 'ARRAY') ?
 	    _complex($P, @{ shift(@_) }) : _simple($P);
 
-    unless (length $P) {
-	if (@_) {
-	    foreach (@_) {
-		$_ = $S . $_;
-	    }
-	} elsif (defined $_) {
-	    $_ = $S . $_;
-	} else {
-	    die "asubstitute: \$_ is undefined: what are you substituting?\n";
-	}
-    }
-
     my ($i, $s, @i, @s, @n, @S);
 
     if (@_) {
@@ -281,7 +342,7 @@ sub asubstitute {
 		_do_substitute(\@n, \@i, \@s, \@S);
 	    }
 	}
-	return $n[0];
+	return $_ = $n[0];
     } else {
 	die "asubstitute: \$_ is undefined: what are you substituting?\n";
     }
@@ -357,7 +418,7 @@ sub adistr {
     my @m = ref $_[0] eq 'ARRAY' ? shift : ();
     if (ref $a0 eq 'ARRAY') {
 	if (ref $a1 eq 'ARRAY') {
-	    my $l0 = length($a0);
+	    my $l0 = length();
 	    return $l0 ? [ map { adist($a0, $_, @m) }
 			  @{$a1} ] :
 		         [ ];
@@ -372,8 +433,12 @@ sub adistr {
 	return     [ map { _adist($a0, $_, @m) / $l0 } @{$a1} ];
     } else {
 	my $l0 = length($a0);
-	return undef unless $l0;
-	return _adist($a0, $a1, @m) / $l0;
+	if (wantarray) {
+	    return map { $l0 ? _adist($a0, $_, @m) / $l0 : undef } ($a1, @_);
+	} else {
+	    return undef unless $l0;
+	    return _adist($a0, $a1, @m) / $l0;
+	}
     }
 }
 
@@ -437,6 +502,16 @@ With this you can emulate errors: typing errorrs, speling errors,
 closely related vocabularies (colour color), genetic mutations (GAG
 ACT), abbreviations (McScot, MacScot).
 
+NOTE: String::Approx has been designed to work with B<strings>, not
+with B<text>.  In other words, when you want to compare things like
+text or source code, consisting of B<words> or B<tokens> and
+B<phrases> and B<sentences>, or B<expressions> and B<statements>,
+you should probably use some other tool than String::Approx, like for
+example the standard UNIX diff(1) tool, or the Algorithm::Diff module
+from CPAN, or if you just want the Levenshtein edit distance
+(explained below), the Text::Levenshtein module from CPAN.  See also
+Text::WagnerFischer and Text::PhraseDistance.
+
 The measure of B<approximateness> is the I<Levenshtein edit distance>.
 It is the total number of "edits": insertions,
 
@@ -455,7 +530,7 @@ transform I<"lead"> into I<"gold">, you need three edits:
 
 	lead gead goad gold
 
-The edit distance of "lead" and "gold" is therefore three.
+The edit distance of "lead" and "gold" is therefore three, or 75%.
 
 =head1 MATCH
 
@@ -494,10 +569,10 @@ element is always a superstring of the pattern.
 
 With the modifiers you can control the amount of approximateness and
 certain other control variables.  The modifiers are one or more
-strings, for example C<"i">, within a string optionally separated by
-whitespace.  The modifiers are inside an anonymous array: the C<[ ]>
-in the syntax are not notational, they really do mean C<[ ]>, for
-example C<[ "i", "2" ]>.  C<["2 i"]> would be identical.
+strings, for example B<"i">, within a string optionally separated by
+whitespace.  The modifiers are inside an anonymous array: the B<[ ]>
+in the syntax are not notational, they really do mean B<[ ]>, for
+example B<[ "i", "2" ]>.  B<["2 i"]> would be identical.
 
 The implicit default approximateness is 10%, rounded up.  In other
 words: every tenth character in the pattern may be an error, an edit.
@@ -507,7 +582,9 @@ modifier like
 	number
 	number%
 
-Examples: C<"3">, C<"15%">.
+Examples: B<"3">, B<"15%">.
+
+Note that C<0%> is not rounded up, it is equal to C<0>.
 
 Using a similar syntax you can separately control the maximum number
 of insertions, deletions, and substitutions by prefixing the numbers
@@ -520,10 +597,10 @@ with I, D, or S, like this:
 	Snumber
 	Snumber%
 
-Examples: C<"I2">, C<"D20%">, C<"S0">.
+Examples: B<"I2">, B<"D20%">, B<"S0">.
 
-You can ignore case (C<"A"> becames equal to C<"a"> and vice versa)
-by adding the C<"i"> modifier.
+You can ignore case (B<"A"> becames equal to B<"a"> and vice versa)
+by adding the B<"i"> modifier.
 
 For example
 
@@ -549,7 +626,7 @@ elements that did match the pattern.  If no inputs are given,
 substitute in the B<$_>.  The replacement can contain magic strings
 B<$&>, B<$`>, B<$'> that stand for the matched string, the string
 before it, and the string after it, respectively.  All the other
-arguments are as in C<amatch()>, plus one additional modifier, C<"g">
+arguments are as in C<amatch()>, plus one additional modifier, B<"g">
 which means substitute globally (all the matches in an element and not
 just the first one, as is the default).
 
@@ -580,13 +657,13 @@ There's also backwards-scanning C<arindex()>.
 	($index, $size)   = aslice("pattern", [ modifiers ])
 	([$i0, $s0], ...) = aslice("pattern", [ modifiers ], @inputs)
 
-Like C<aindex()> but returns also the size of the match.  If the
-match fails, returns an empty list (when matching against C<$_>) or
-an empty anonymous list corresponding to the particular input.
+Like C<aindex()> but returns also the size (length) of the match.
+If the match fails, returns an empty list (when matching against C<$_>)
+or an empty anonymous list corresponding to the particular input.
 
-Note that the size of the match will very probably be something
-you did not expect (such as longer than the pattern).  This may
-or may not be fixed in future releases.
+Note that the size of the match will very probably be something you
+did not expect (such as longer than the pattern, or a negative
+number).  This may or may not be fixed in future releases.
 
 If the modifier
 
@@ -609,7 +686,7 @@ Return the I<edit distance> or distances between the pattern and the
 input or inputs.  Zero edit distance means exact match.  (Remember
 that the match can 'float' in the inputs, the match is a substring
 match.)  If the pattern is longer than the input or inputs, the
-returned distance or distance is or are negative.
+returned distance or distances is or are negative.
 
 	use String::Approx 'adistr';
 
@@ -623,6 +700,59 @@ match can 'float' in the inputs, the match is a substring match.)  If
 the pattern is longer than the input or inputs, the returned distance
 or distances is or are negative.
 
+You can use adist() or adistr() to sort the inputs according to their
+approximateness:
+
+	my %d;
+	@d{@inputs} = map { abs } adistr("pattern", @inputs);
+	my @d = sort { $d{$a} <=> $d{$b} } @inputs;
+
+Now C<@d> contains the inputs, the most like C<"pattern"> first.
+
+=head1 CONTROLLING THE CACHE
+
+C<String::Approx> maintains a LU (least-used) cache that holds the
+'matching engines' for each instance of a I<pattern+modifiers>.  The
+cache is intended to help the case where you match a small set of
+patterns against a large set of string.  However, the more engines you
+cache the more you eat memory.  If you have a lot of different
+patterns or if you have a lot of memory to burn, you may want to
+control the cache yourself.  For example, allowing a larger cache
+consumes more memory but probably runs a little bit faster since the
+cache fills (and needs flushing) less often.
+
+The cache has two parameters: I<max> and I<purge>.  The first one
+is the maximum size of the cache and the second one is the cache
+flushing ratio: when the number of cache entries exceeds I<max>,
+I<max> times I<purge> cache entries are flushed.  The default
+values are 1000 and 0.75, respectively, which means that when
+the 1001st entry would be cached, 750 least used entries will
+be removed from the cache.  To access the parameters you can
+use the calls
+
+	$now_max = String::Approx::cache_max();
+	String::Approx::cache_max($new_max);
+
+	$now_purge = String::Approx::cache_purge();
+	String::Approx::cache_purge($new_purge);
+
+	$limit = String::Approx::cache_n_purge();
+
+To be honest, there are actually B<two> caches: the first one is used
+far the patterns with no modifiers, the second one for the patterns
+with pattern modifiers.  Using the standard parameters you will
+therefore actually cache up to 2000 entries.  The above calls control
+both caches for the same price.
+
+To disable caching completely use
+
+	String::Approx::cache_disable();
+
+Note that this doesn't flush any possibly existing cache entries,
+to do that use
+
+	String::Approx::cache_flush_all();
+
 =head1 NOTES
 
 Because matching is by I<substrings>, not by whole strings, insertions
@@ -630,7 +760,7 @@ and substitutions produce often very similar results: "abcde" matches
 "axbcde" either by insertion B<or> substitution of "x".
 
 The maximum edit distance is also the maximum number of edits.
-That is, the C<"I2"> in
+That is, the B<"I2"> in
 
 	amatch("abcd", ["I2"])
 
@@ -663,7 +793,7 @@ or the both the modifiers
 	"initial_position=24"
 	"position_range=10"
 
-By setting the C<"position_range"> to be zero you can limit
+By setting the B<"position_range"> to be zero you can limit
 (anchor) the operation to happen only once (if a match is possible)
 at the position.
 
@@ -704,11 +834,11 @@ scalable algorithm written in C is used.
 The string matched and substituted is now always stingy, as short
 as possible.  It used to be as long as possible.  This is an unfortunate
 change stemming from switching the matching algorithm.  Example: with
-edit distance of two and substituting for C<"word"> from C<"cork"> and
-C<"wool"> previously did match C<"cork"> and C<"wool">.  Now it does
-match C<"or"> and C<"wo">.  As little as possible, or, in other words,
+edit distance of two and substituting for B<"word"> from B<"cork"> and
+B<"wool"> previously did match B<"cork"> and B<"wool">.  Now it does
+match B<"or"> and B<"wo">.  As little as possible, or, in other words,
 with as much approximateness, as many edits, as possible.  Because
-there is no I<need> to match the C<"c"> of C<"cork">, it is not matched.
+there is no I<need> to match the B<"c"> of B<"cork">, it is not matched.
 
 =item no more C<aregex()> because regular expressions are no more used
 
@@ -718,13 +848,19 @@ there is no I<need> to match the C<"c"> of C<"cork">, it is not matched.
 
 =head1 ACKNOWLEDGEMENTS
 
-The following people have provided with valuable test cases and other
-feedback: Jared August, Steve A. Chervitz, Alberto Fontaneda, Dmitrij
-Frishman, Lars Gregersen, Kevin Greiner, Mike Hanafey, Ricky Houghton,
-Helmut Jarausch, Ben Kennedy, Mark Land, Sergey Novoselov, Andy Oram,
-Eric Promislow, Stefan Ram, Stewart Russell, Slaven Rezic, Chris
-Rosin, Ilya Sandler, Bob J.A. Schijvenaars, Ross Smith, Greg Ward,
-Rick Wise.
+The following people have provided valuable test cases, documentation
+clarifications, and other feedback:
+
+Jared August, Arthur Bergman, Anirvan Chatterjee, Steve A. Chervitz,
+Aldo Calpini, David Curiel, Teun van den Dool, Alberto Fontaneda,
+Rob Fugina, Dmitrij Frishman, Lars Gregersen, Kevin Greiner,
+B. Elijah Griffin, Mike Hanafey, Mitch Helle, Ricky Houghton,
+'idallen', Helmut Jarausch, Damian Keefe, Ben Kennedy, Craig Kelley,
+Franz Kirsch, Dag Kristian, Mark Land, J. D. Laub, Tim Maher,
+Juha Muilu, Sergey Novoselov, Andy Oram, Ji Y Park, Eric Promislow,
+Nikolaus Rath, Stefan Ram, Dag Kristian Rognlien, Stewart Russell,
+Slaven Rezic, Chris Rosin, Pasha Sadri, Ilya Sandler, Bob J.A. Schijvenaars,
+Ross Smith, Frank Tobin, Greg Ward, Rich Williams, Rick Wise.
 
 The matching algorithm was developed by Udi Manber, Sun Wu, and Burra
 Gopal in the Department of Computer Science, University of Arizona.
@@ -733,4 +869,15 @@ Gopal in the Department of Computer Science, University of Arizona.
 
 Jarkko Hietaniemi <jhi@iki.fi>
 
+=head1 COPYRIGHT AND LICENSE
+
+Copyright 2001-2005 by Jarkko Hietaniemi
+
+This library is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself.
+
+Furthermore: no warranties or obligations of any kind are given, and
+the separate file F<COPYRIGHT> must be included intact in all copies
+and derived materials.
+
 =cut
